import click
import logging
import os
import subprocess
import sys
import yaml

logger = logging.getLogger(__name__)

logo = """
.  . .  .  . .  .  . ..;tS@@@@X%;:.  .  . .  .  
   .       .       ;X@888888888888@88@tSS;    .   
     .  .    .  .;S   S88t         8S888SS8:    . 
 .       .    .@888 tS;t:             88888;t.    
   .  .   . ;@ @8t ::.X8 8 8:        .: 8888@8;.  
  .    ..ttX  @:;   ..888888X          : t8@88@  .
    .  .8@S8X; .88;  :888888;:  88t  .    88888   
  .   ;S88X%  .88888X.888888:t88888%.    :888888 .
    .XS8@8% ..88888888%8888888888888t: . ..8888S  
  . t  8 t   tS888888888@@8888888@88. .   :88@8X  
   :8 88    .:;  888@8888X888@ 888%.  .. .8888SX .
  .%8X8;         888@888X8888@888:   .  . 88888t. 
  t@88S;  .  .@88888888X8@8888@88888;  . 888888   
  X8888     ..8888888S8X888 88888888t.. t@888@   .
  X888Xt .     8888S8X8888@8 8888888:  ;8888S .   
 .S@8888% .   t%88@.:.888888%   88@:..8@8888:   . 
  t@8888;     .....  .@88888S   ;: ..8888SS.: .   
  .888888;.   ..     :8888 8    .. .8@8@88888:   .
  : 88888@:@:.   .  ..          t:.;t888SS 8XS  . 
   : ; 888888 8%:  .      .S88888888888@8%S@;     
     :.888888888@88888888%88888888S@888888;  . .  
  .  ...t888888888888888888888888; 888 888@..    .
   .   .;;  %8888 8888@8888:  %X: .:t;:   . . .   
     .       ;S% :     t:;                   .  . 


       """

PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))

def install_playbook(playbook_path, params=None):
    # This can be called to execute an Ansible playbook.
    click.secho('Installing from %s' % playbook_path, bold=True)
    cmd = ['ansible-playbook', '--connection=local',               
           '--inventory', '127.0.0.1,', '{}'.format(playbook_path)
    ]
    # Inject vars into every playbook
    cmd.extend(['--extra-vars=@config.yml'])
    if params:
        cmd.extend(list(params))
    data = subprocess.Popen(cmd, cwd=os.getenv('AMANITA_PREFIX'), 
                           stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT)
    for line in iter(data.stdout.readline, b''):
        click.echo(line.decode().strip('\n'))
    exitcode = data.wait()
    if exitcode != 0:
        click.secho('Template exit code: {}'.format(exitcode),
                    bold=True, fg='red')
        sys.exit(exitcode)


class MyCLI(click.MultiCommand):

    def __init__(self, *args, **kwargs):
        self.folder = kwargs.pop('folder')
        self.folder_path = os.path.join(PACKAGE_DIR, self.folder)
        super(MyCLI, self).__init__(*args, **kwargs)

    def list_custom_commands(self, ctx):
        rv = []
        for filename in os.listdir(self.folder_path):
            if filename.endswith('.py') and filename.startswith('cmd_'):
                filename = filename.replace('_', '-')
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def list_playbooks(self, ctx):
        rv = []
        for filename in os.listdir(self.folder_path):
            if filename.endswith('.yml') and not filename.startswith('_'):
                rv.append(filename[:-4])
        rv.sort()
        return rv

    def list_commands(self, ctx):
        return self.list_custom_commands(ctx) + self.list_playbooks(ctx)


    def get_custom_command(self, ctx, name):
        try:
            name = name.replace('-', '_')
            mod = __import__('amanita_cli.' + self.folder + '.cmd_' + name,
                             None, None, [self.folder])
        except ImportError as e:
            click.secho('Could not import {}: {}'.format(name, e),
                        bold=True, fg='red')
            sys.exit(2)
        return mod.cli

    def get_command(self, ctx, name):
        files = os.listdir(self.folder_path)
        file_name = None
        for file in files:
            if file == name + '.yml':
                file_name = file
                break            
        if not file_name:
            return self.get_custom_command(ctx, name)        
        # Now define an ansible playbook command
        @click.command(context_settings=dict(ignore_unknown_options=True),
                       help='Ansible template')
        @click.argument('params', nargs=-1, type=click.UNPROCESSED)
        def cmd(params=None):
            file_path = os.path.join(self.folder_path, file_name)
            if os.path.isdir(file_path):
                file_path = os.path.join(file_path, 'main.yml')            
            install_playbook(file_path, params=params)
        return cmd


def config_load():
    amanita_dir = os.getenv('AMANITA_PREFIX')
    if not amanita_dir:
        if os.path.exists(os.path.expanduser('~/.amanita')):
            amanita_dir = open(
                    os.path.expanduser('~/.amanita')).read().strip('\n')
        else:
            amanita_dir = '/srv/amanita'
            open(os.path.expanduser('~/.amanita'), 'w').write(
                                                '{}\n'.format(amanita_dir))
        os.environ['AMANITA_PREFIX'] = amanita_dir
    else:
        # Env var set, check that it's in ~/.amanita
        if not os.path.exists(os.path.expanduser('~/.amanita')):
            open(os.path.expanduser('~/.amanita'), 'w').write(
                                                '{}\n'.format(amanita_dir))
        elif open(os.path.expanduser('~/.amanita')).read(
                                            ).strip('\n') != amanita_dir:
            # Overwrite ~/.amanita with prefix set
            open(os.path.expanduser('~/.amanita'), 'w').write(
                                                '{}\n'.format(amanita_dir))            
    # Create AMANITA top dir if not found
    if not os.path.isdir(amanita_dir):
        os.makedirs(amanita_dir, mode=0o711)
    #
    config_path = os.path.join(amanita_dir, 'config.yml')    
    if not os.path.isfile(config_path):
        # Read fefault config...
        config = yaml.load(
            open(os.path.join(PACKAGE_DIR, 'config.default.yml')),
            yaml.SafeLoader)
        config['AMANITA_PREFIX'] = amanita_dir
        # ... and copy it to AMANITA top dir
        open(config_path, 'w').write(yaml.dump(config))        
        click.secho('Creating Amanita config at {}'.format(config_path),
                    bold=True, fg='green')
    else:
        config = yaml.load(open(config_path), yaml.SafeLoader)
        config['AMANITA_PREFIX'] = amanita_dir
    for var in config:
        if not os.getenv(var):
            # Populate environent if var is not already there
            if type(config[var]) is str:
                os.environ[var] = config[var]    
    return config

def config_save(config):
    amanita_dir = os.getenv('AMANITA_PREFIX')
    config_path = os.path.join(amanita_dir, 'config.yml')
    open(config_path, 'w').write(yaml.dump(config))


@click.group()
@click.pass_context
def cli(ctx):
    """
    Amanita Telefonica Command Center. 
    Here you manage Amanita services deployed on current machine.
    Please select your command....
    """
    # Load installation configuration
    config = config_load()
    # Change working directory to Amanita root dir.
    os.chdir(config['AMANITA_PREFIX'])

# Find modules
for f in os.listdir(PACKAGE_DIR):    
    folder_path = os.path.join(PACKAGE_DIR, f)
    description_path = os.path.join(folder_path, '.module')
    if not os.path.isdir(folder_path) or not os.path.isfile(description_path):
        continue
    mod_name = f.replace('_', '-')    
    addon = MyCLI(folder=f, name=mod_name,
                  help=open(description_path).read().strip('\n'))
    cli.add_command(addon)

if __name__ == '__main__':
    cli()
