import click
import eventlet
import odoorpc
import socket
import subprocess
from amanita_cli.main import config_load, config_save
from amanita_cli.main import cli as main_cli

config = config_load()

@click.command('start')
@click.option('--install', is_flag=True, 
              help='Run installation scripts before start.')
@click.option('--function', help='Function to invoke, see source code for details.')
@click.option('--ommit-module', help='Do not deal with module specified.',
              multiple=True)
@click.pass_context
def cli(ctx, install, function, ommit_module):
    """Run demo of all Amanita Telefonica services on this machine. This includes
    installing and running:

    \b
    - PostgreSQL database
    - RabbitMQ server
    - Supervisor daemon
    - Odoo & Odoo-broker services
    - Asterisk, Asterisk-broker, console services
    """
    if function:
        f = globals()[function]
        return f(ctx)
    if not install:
        start(ctx)
    else:
        install_all(ctx)
        start(ctx)


def start(ctx):
    pass


def modules_install(ctx):
    modules = ['postgres', 'rabbitmq', 'caddy', 'asterisk', 'supervisor', 'odoo',
               'asterisk-broker', 'odoo-broker', 'console-broker']
    for mod in modules:
        if mod in  ctx.params['ommit_module']:
            click.secho('Omitting {}'.format(mod), 
                        bold=True, bg='black', fg='blue')
            continue
        click.secho('Installing %s' % mod, bold=True, bg='black', fg='white')
        l1 = main_cli.get_command(ctx, mod)
        l2 = l1.get_command(ctx, 'install')
        ctx.invoke(l2)


def odoo_addons_install(ctx):
    click.secho('Installing Odoo addon amanita_base.', bold=True,
                bg='black', fg='white')
    oa = main_cli.get_command(ctx, 'odoo-addons')
    dw = oa.get_command(ctx, 'download')
    ctx.invoke(dw, name='amanita_base')
    ab = oa.get_command(ctx, 'install')
    ctx.invoke(ab, name='amanita_base')
    # Now restart Odoo


def asterisk_baseconf_install(ctx):
    click.secho('Installing Asterisk base .conf files.', bold=True,
                bg='black', fg='white')
    ab = main_cli.get_command(ctx, 'asterisk-broker')
    bc = ab.get_command(ctx, 'baseconf')
    ctx.invoke(bc)
    # Now restart Asterisk if it is already running
    try:
        subprocess.check_output('asterisk -rx "core restart now"', shell=True)
    except subprocess.CalledProcessError as e:
        click.echo(e.output)


def enable_supervisor_services(ctx):
    for s in ['odoo', 'asterisk-broker', 'odoo-broker', 'console-broker']:
        sup = main_cli.get_command(ctx, 'supervisor')
        enable = sup.get_command(ctx, 'enable')
        click.secho('Enable supervisor service {}.'.format(s), bold=True,
                    bg='black', fg='white')
        ctx.invoke(enable, service=s)
    click.secho('Start supervisor services.', bold=True,
                    bg='black', fg='white')
    apply = sup.get_command(ctx, 'apply')
    ctx.invoke(apply)


def load_data(ctx):
    click.secho('Configure local data.', bold=True, bg='black', fg='white')    
    odoo = odoorpc.ODOO(config['ODOO_HOST'], port=int(config['ODOO_PORT']))
    odoo.login(config['ODOO_DB'], config['ODOO_USER'], config['ODOO_PASS'])
    odoo.env['ir.module.module'].update_list()
    try:
        odoo.env['amanita_base.server'].create(
                    {'name': 'local', 'system_name': 'asterisk'})
    except Exception as e:
        if 'This System Name is already used' in str(e):
            pass
    odoo.env['res.config.settings'].set_amanita_param(
                                            'amqp_uri', config['AMQP_URI'])
    odoo.env['res.config.settings'].set_amanita_param(
                            'console_uri', 'ws://127.0.0.1/console-ws/')


def congrats(ctx):
    click.secho('Congratulations! Amanita has been spawned here :-)',
                bold=True, fg='white')
    click.secho('Try this: http://{}/ (this IP was returned by socket functions) '.format(
                socket.gethostbyname(socket.gethostname())),
                bold=True, fg='red')


def install_all(ctx):
    # Install Amanita components
    modules_install(ctx)
    # Now install Odoo addons
    odoo_addons_install(ctx)
    # Now install Asterisk base conf
    asterisk_baseconf_install(ctx)
    # Now enable all services on supervisor
    enable_supervisor_services(ctx)
    # Update amanita_base AMQP URI and create local Asterisk server
    eventlet.sleep(10) # Wait 10 seconds for Odoo to come up...
    while True:
        try:
            load_data(ctx)
            break
        except Exception as e:
            if 'Connection refused' in str(e):
                click.secho('Odoo connection refused, '
                            'waiting 10 seconds for stand up...')
                eventlet.sleep(10)
    congrats(ctx)
